@@ -1,7 +1,9 @@
 #ifndef _DBI_H
 #define _DBI_H
 
+#include <stdio.h>
 #include "dbiset.h"
+#include <rpm/rpmtag.h>
 
 /* XXX: make this backend-specific, eliminate or something... */
 #define	_USE_COPY_LOAD
@@ -129,6 +131,9 @@ void dbSetFSync(rpmdb rdb, int enable);
 RPM_GNUC_INTERNAL
 int dbCtrl(rpmdb rdb, dbCtrlOp ctrl);
 
+RPM_GNUC_INTERNAL
+void dbShowRC(FILE* fp);
+
 /** \ingroup dbi
  * Return new configured index database handle instance.
  * @param rdb		rpm database
