@@ -44,13 +44,6 @@ static void rpm_match3(sqlite3_context *sctx, int argc, sqlite3_value **argv)
     sqlite3_result_int(sctx, match);
 }
 
-static void errCb(void *data, int err, const char *msg)
-{
-    rpmdb rdb = data;
-    rpmlog(RPMLOG_WARNING, "%s: %s: %s\n",
-		rdb->db_descr, sqlite3_errstr(err), msg);
-}
-
 static int dbiCursorReset(dbiCursor dbc)
 {
     if (dbc->stmt) {
@@ -135,7 +128,7 @@ static int sqlite_init(rpmdb rdb, const char * dbhome)
     if (rdb->db_dbenv == NULL) {
 	dbfile = rpmGenPath(dbhome, rdb->db_ops->path, NULL);
 	sqlite3 *sdb = NULL;
-	int xx, flags = 0;
+	int xx = SQLITE_CANTOPEN, flags = 0;
 	int retry_open = 1;
 	if ((rdb->db_mode & O_ACCMODE) == O_RDONLY)
 	    flags |= SQLITE_OPEN_READONLY;
@@ -170,7 +163,6 @@ static int sqlite_init(rpmdb rdb, const char * dbhome)
 	 * the "database is locked" errors at every cost
 	 */
 	sqlite3_busy_timeout(sdb, 10000);
-	sqlite3_config(SQLITE_CONFIG_LOG, errCb, rdb);
 
 	sqlexec(sdb, "PRAGMA secure_delete = OFF");
 	sqlexec(sdb, "PRAGMA case_sensitive_like = ON");
