@@ -8,7 +8,6 @@
 #include <rpm/header.h>
 #include <rpm/rpmtag.h>
 #include <rpm/rpmstring.h>
-#include <rpm/rpmpgp.h>
 #include "lib/misc.h"		/* format function protos */
 
 #include "debug.h"
@@ -773,6 +772,12 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 
 	    if (isxml) {
 		const char * tagN = rpmTagGetName(spft->u.tag.tag);
+		char *tagval = NULL;
+
+		if (rstreq(tagN, "(unknown)")) {
+		    rasprintf(&tagval, "[%u]", spft->u.tag.tag);
+		    tagN = tagval;
+		}
 
 		need = sizeof("  <rpmTag name=\"\">\n") - 1;
 		if (tagN != NULL)
@@ -783,6 +788,8 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 		    te = stpcpy(te, tagN);
 		te = stpcpy(te, "\">\n");
 		hsa->vallen += (te - t);
+
+		free(tagval);
 	    }
 
 	    t = hsaReserve(hsa, need);
