@@ -4,9 +4,12 @@
 
 #include "system.h"
 
+#include <string.h>
+
 #include <rpm/rpmlog.h>
 #include <rpm/rpmfileutil.h>
 #include <rpm/argv.h>
+#include <rpm/rpmstring.h>
 
 #include "lib/manifest.h"
 
@@ -84,7 +87,8 @@ rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, char *** argvPtr)
 	}
 
 	/* Skip comments. */
-	if ((se = strchr(s, '#')) != NULL) *se = '\0';
+	if (*s == '#')
+	    continue;
 
 	/* Trim white space. */
 	se = s + strlen(s);
