@@ -1099,7 +1099,7 @@ static char * mireDup(rpmTagVal tag, rpmMireMode *modep,
 	nb = strlen(pattern) + sizeof("^$");
 
 	/* Find no. of bytes needed for pattern. */
-	/* periods and plusses are escaped, splats become '.*' */
+	/* periods, plusses and carets are escaped, splats become '.*' */
 	c = '\0';
 	brackets = 0;
 	for (s = pattern; *s != '\0'; s++) {
@@ -1127,7 +1127,7 @@ static char * mireDup(rpmTagVal tag, rpmMireMode *modep,
 
 	if (pattern[0] != '^') *t++ = '^';
 
-	/* Copy pattern, escaping periods, prefixing splats with period. */
+	/* Copy pattern, escaping periods, plusses and carets, and prefixing splats with period. */
 	c = '\0';
 	brackets = 0;
 	for (s = pattern; *s != '\0'; s++, t++) {
@@ -1196,6 +1196,10 @@ int rpmdbSetIteratorRE(rpmdbMatchIterator mi, rpmTagVal tag,
 	free(t);
      }
 
+    /* Handle missing epoch, see mireSkip() */
+    if (tag == RPMTAG_EPOCH && pattern == NULL)
+	pattern = "0";
+
     if (mi == NULL || pattern == NULL)
 	return rc;
 
@@ -2466,8 +2470,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	    /* let's sanity check this record a bit, otherwise just skip it */
 	    if (!(headerIsEntry(h, RPMTAG_NAME) &&
 		headerIsEntry(h, RPMTAG_VERSION) &&
-		headerIsEntry(h, RPMTAG_RELEASE) &&
-		headerIsEntry(h, RPMTAG_BUILDTIME)))
+		headerIsEntry(h, RPMTAG_RELEASE)))
 	    {
 		rpmlog(RPMLOG_ERR,
 			_("header #%u in the database is bad -- skipping.\n"),
