@@ -11,6 +11,7 @@
 #include <rpm/rpmstring.h>
 #include <rpm/rpmmacro.h>	/* XXX rpmCleanPath */
 #include <rpm/rpmds.h>
+#include <rpm/rpmbase64.h>
 #include <errno.h>
 #include <fcntl.h>
 
@@ -20,7 +21,6 @@
 #include "lib/fsm.h"	/* rpmpsm stuff for now */
 #include "lib/rpmug.h"
 #include "rpmio/rpmio_internal.h"       /* fdInit/FiniDigest */
-#include "rpmio/rpmbase64.h"
 
 #include "debug.h"
 
@@ -55,6 +55,9 @@ struct rpmfi_s {
     int intervalStart;		/*!< Start of iterating interval. */
     int intervalEnd;		/*!< End of iterating interval. */
 
+    rpmfiChdirCb onChdir;	/*!< Callback for directory changes */
+    void *onChdirData;		/*!< Caller private callback data */
+
     rpmfiles files;		/*!< File info set */
     rpmcpio_t archive;		/*!< Archive with payload */
     unsigned char * found;	/*!< Bit field of files found in the archive */
@@ -116,7 +119,7 @@ struct rpmfiles_s {
     struct fingerPrint_s * fps;	/*!< File fingerprint(s). */
 
     int digestalgo;		/*!< File digest algorithm */
-    int signaturelength;	/*!< File signature length */
+    uint32_t *signatureoffs;	/*!< File signature offsets */
     int veritysiglength;	/*!< Verity signature length */
     uint16_t verityalgo;	/*!< Verity algorithm */
     unsigned char * digests;	/*!< File digests in binary. */
@@ -303,11 +306,16 @@ rpm_count_t rpmfiDC(rpmfi fi)
     return (fi != NULL ? rpmfilesDC(fi->files) : 0);
 }
 
-#ifdef	NOTYET
-int rpmfiDI(rpmfi fi)
+int rpmfiSetOnChdir(rpmfi fi, rpmfiChdirCb cb, void *data)
 {
+    int rc = -1;
+    if (fi != NULL) {
+	fi->onChdir = cb;
+	fi->onChdirData = data;
+	rc = 0;
+    }
+    return rc;
 }
-#endif
 
 int rpmfiFX(rpmfi fi)
 {
@@ -319,9 +327,17 @@ int rpmfiSetFX(rpmfi fi, int fx)
     int i = -1;
 
     if (fi != NULL && fx >= 0 && fx < rpmfilesFC(fi->files)) {
+	int dx = fi->j;
 	i = fi->i;
 	fi->i = fx;
 	fi->j = rpmfilesDI(fi->files, fi->i);
+	i = fi->i;
+
+	if (fi->j != dx && fi->onChdir) {
+	    int chrc = fi->onChdir(fi, fi->onChdirData);
+	    if (chrc < 0)
+		i = chrc;
+	}
     }
     return i;
 }
@@ -333,13 +349,7 @@ int rpmfiDX(rpmfi fi)
 
 int rpmfiSetDX(rpmfi fi, int dx)
 {
-    int j = -1;
-
-    if (fi != NULL && dx >= 0 && dx < rpmfiDC(fi)) {
-	j = fi->j;
-	fi->j = dx;
-    }
-    return j;
+    return -1;
 }
 
 int rpmfilesDI(rpmfiles fi, int ix)
@@ -568,7 +578,7 @@ char * rpmfiFDigestHex(rpmfi fi, int *algo)
     char *fdigest = NULL;
     const unsigned char *digest = rpmfiFDigest(fi, algo, &diglen);
     if (digest) {
-	fdigest = pgpHexStr(digest, diglen);
+	fdigest = rpmhex(digest, diglen);
     }
     return fdigest;
 }
@@ -578,10 +588,15 @@ const unsigned char * rpmfilesFSignature(rpmfiles fi, int ix, size_t *len)
     const unsigned char *signature = NULL;
 
     if (fi != NULL && ix >= 0 && ix < rpmfilesFC(fi)) {
-	if (fi->signatures != NULL)
-	    signature = fi->signatures + (fi->signaturelength * ix);
+	size_t slen = 0;
+	if (fi->signatures != NULL && fi->signatureoffs != NULL) {
+	    uint32_t off = fi->signatureoffs[ix];
+	    slen = fi->signatureoffs[ix+1] - off;
+	    if (slen > 0)
+		signature = fi->signatures + off;
+	}
 	if (len)
-	    *len = fi->signaturelength;
+	    *len = slen;
     }
     return signature;
 }
@@ -735,7 +750,7 @@ uint32_t rpmfilesFLinks(rpmfiles fi, int ix, const int ** files)
 
 uint32_t rpmfiFLinks(rpmfi fi, const int ** files)
 {
-    return rpmfilesFLinks(fi->files, fi ? fi->i : -1, files);
+    return rpmfilesFLinks(fi ? fi->files : NULL, fi ? fi->i : -1, files);
 }
 
 uint32_t rpmfilesFNlink(rpmfiles fi, int ix)
@@ -866,15 +881,8 @@ int rpmfiNext(rpmfi fi)
 	    next = fi->next(fi);
 	} while (next == RPMERR_ITER_SKIP);
 
-	if (next >= 0 && next < rpmfilesFC(fi->files)) {
-	    fi->i = next;
-	    fi->j = rpmfilesDI(fi->files, fi->i);
-	} else {
-	    fi->i = -1;
-	    if (next >= 0) {
-		next = -1;
-	    }
-	}
+	if (next >= 0)
+	    next = rpmfiSetFX(fi, next);
     }
     return next;
 }
@@ -893,28 +901,12 @@ rpmfi rpmfiInit(rpmfi fi, int fx)
 
 int rpmfiNextD(rpmfi fi)
 {
-    int j = -1;
-
-    if (fi != NULL && ++fi->j >= 0) {
-	if (fi->j < rpmfilesDC(fi->files))
-	    j = fi->j;
-	else
-	    fi->j = -1;
-    }
-
-    return j;
+    return -1;
 }
 
 rpmfi rpmfiInitD(rpmfi fi, int dx)
 {
-    if (fi != NULL) {
-	if (dx >= 0 && dx < rpmfilesFC(fi->files))
-	    fi->j = dx - 1;
-	else
-	    fi = NULL;
-    }
-
-    return fi;
+    return NULL;
 }
 
 rpmFileTypes rpmfiWhatis(rpm_mode_t mode)
@@ -1277,6 +1269,7 @@ rpmfiles rpmfilesFree(rpmfiles fi)
 	fi->flangs = _free(fi->flangs);
 	fi->digests = _free(fi->digests);
 	fi->signatures = _free(fi->signatures);
+	fi->signatureoffs = _free(fi->signatureoffs);
 	fi->veritysigs = _free(fi->veritysigs);
 	fi->fcaps = _free(fi->fcaps);
 
@@ -1506,6 +1499,48 @@ err:
     return;
 }
 
+/*
+ * Convert a tag of variable len hex strings to binary presentation,
+ * accessed via offsets to a contiguous binary blob. Empty values
+ * are represented by identical consequtive offsets. The offsets array
+ * always has one extra element to allow calculating the size of the
+ * last element.
+ */
+static uint8_t *hex2binv(Header h, rpmTagVal tag, rpm_count_t num,
+			uint32_t **offsetp)
+{
+    struct rpmtd_s td;
+    uint8_t *bin = NULL;
+    uint32_t *offs = NULL;
+
+    if (headerGet(h, tag, &td, HEADERGET_MINMEM) && rpmtdCount(&td) == num) {
+	const char *s;
+	int i = 0;
+	uint8_t *t = bin = xmalloc(((rpmtdSize(&td) / 2) + 1));
+	offs = xmalloc((num + 1) * sizeof(*offs));
+
+	while ((s = rpmtdNextString(&td))) {
+	    uint32_t slen = strlen(s);
+	    uint32_t len = slen / 2;
+	    if (slen % 2) {
+		bin = rfree(bin);
+		offs = rfree(offs);
+		goto exit;
+	    }
+	    offs[i] = t - bin;
+	    for (int j = 0; j < len; j++, t++, s += 2)
+		*t = (rnibble(s[0]) << 4) | rnibble(s[1]);
+	    i++;
+	}
+	offs[i] = t - bin;
+	*offsetp = offs;
+    }
+
+exit:
+    rpmtdFreeData(&td);
+    return bin;
+}
+
 /* Convert a tag of hex strings to binary presentation */
 static uint8_t *hex2bin(Header h, rpmTagVal tag, rpm_count_t num, size_t len)
 {
@@ -1522,6 +1557,10 @@ static uint8_t *hex2bin(Header h, rpmTagVal tag, rpm_count_t num, size_t len)
 		t += len;
 		continue;
 	    }
+	    if (strlen(s) != len * 2) {
+		bin = rfree(bin);
+		break;
+	    }
 	    for (int j = 0; j < len; j++, t++, s += 2)
 		*t = (rnibble(s[0]) << 4) | rnibble(s[1]);
 	}
@@ -1547,7 +1586,7 @@ static uint8_t *base2bin(Header h, rpmTagVal tag, rpm_count_t num, int *len)
     size_t *lengths = xcalloc(num, sizeof(size_t));
     const char *s;
 
-    if (headerGet(h, tag, &td, HEADERGET_MINMEM) && rpmtdCount(&td) != num)
+    if (!headerGet(h, tag, &td, HEADERGET_MINMEM) || rpmtdCount(&td) != num)
 	goto out;
 
     while ((s = rpmtdNextString(&td))) {
@@ -1574,8 +1613,10 @@ static uint8_t *base2bin(Header h, rpmTagVal tag, rpm_count_t num, int *len)
 	t = bin = xcalloc(num, maxlen);
 
 	for (i = 0; i < num; i++) {
-	    memcpy(t, arr[i], lengths[i]);
-	    free(arr[i]);
+	    if (arr[i]) {
+		memcpy(t, arr[i], lengths[i]);
+		free(arr[i]);
+	    }
 	    t += maxlen;
 	}
 	*len = maxlen;
@@ -1635,7 +1676,7 @@ static int rpmfilesPopulate(rpmfiles fi, Header h, rpmfiFlags flags)
 	fi->flangs = tag2pool(fi->pool, h, RPMTAG_FILELANGS, totalfc);
 
     /* See if the package has non-md5 file digests */
-    fi->digestalgo = PGPHASHALGO_MD5;
+    fi->digestalgo = RPM_HASH_MD5;
     if (headerGet(h, RPMTAG_FILEDIGESTALGO, &digalgo, HEADERGET_MINMEM)) {
 	uint32_t *algo = rpmtdGetUint32(&digalgo);
 	/* Hmm, what to do with unknown digest algorithms? */
@@ -1654,9 +1695,8 @@ static int rpmfilesPopulate(rpmfiles fi, Header h, rpmfiFlags flags)
     fi->signatures = NULL;
     /* grab hex signatures from header and store in binary format */
     if (!(flags & RPMFI_NOFILESIGNATURES)) {
-	fi->signaturelength = headerGetNumber(h, RPMTAG_FILESIGNATURELENGTH);
-	fi->signatures = hex2bin(h, RPMTAG_FILESIGNATURES,
-				 totalfc, fi->signaturelength);
+	fi->signatures = hex2binv(h, RPMTAG_FILESIGNATURES,
+				 totalfc, &fi->signatureoffs);
     }
 
     fi->veritysigs = NULL;
@@ -1764,9 +1804,9 @@ static rpmfi initIter(rpmfiles files, int itype, int link)
     if (files && itype>=0 && itype<=RPMFILEITERMAX) {
 	fi = xcalloc(1, sizeof(*fi)); 
 	fi->i = -1;
+	fi->j = -1;
 	fi->files = link ? rpmfilesLink(files) : files;
 	fi->next = nextfuncs[itype];
-	fi->i = -1;
 	if (itype == RPMFI_ITER_BACK) {
 	    fi->i = rpmfilesFC(fi->files);
 	} else if (itype >=RPMFI_ITER_READ_ARCHIVE
@@ -1948,22 +1988,22 @@ const char * rpmfiOFN(rpmfi fi)
 
 const unsigned char * rpmfiFDigest(rpmfi fi, int *algo, size_t *len)
 {
-    return rpmfilesFDigest(fi->files, fi ? fi->i : -1, algo, len);
+    return rpmfilesFDigest(fi ? fi->files : NULL, fi ? fi->i : -1, algo, len);
 }
 
 const unsigned char * rpmfiFSignature(rpmfi fi, size_t *len)
 {
-    return rpmfilesFSignature(fi->files, fi ? fi->i : -1, len);
+    return rpmfilesFSignature(fi ? fi->files : NULL, fi ? fi->i : -1, len);
 }
 
 const unsigned char * rpmfiVSignature(rpmfi fi, size_t *len, uint16_t *algo)
 {
-    return rpmfilesVSignature(fi->files, fi ? fi->i : -1, len, algo);
+    return rpmfilesVSignature(fi ? fi->files : NULL, fi ? fi->i : -1, len, algo);
 }
 
 uint32_t rpmfiFDepends(rpmfi fi, const uint32_t ** fddictp)
 {
-    return rpmfilesFDepends(fi->files,  fi ? fi->i : -1, fddictp);
+    return rpmfilesFDepends(fi ? fi->files : NULL,  fi ? fi->i : -1, fddictp);
 }
 
 int rpmfiStat(rpmfi fi, int flags, struct stat *sb)
@@ -1983,7 +2023,8 @@ int rpmfiStat(rpmfi fi, int flags, struct stat *sb)
 
 int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
 {
-    return rpmfilesCompare(afi->files, afi ? afi->i : -1, bfi->files, bfi ? bfi->i : -1);
+    return rpmfilesCompare(afi ? afi->files : NULL, afi ? afi->i : -1,
+			   bfi ? bfi->files : NULL, bfi ? bfi->i : -1);
 }
 
 rpmVerifyAttrs rpmfiVerify(rpmfi fi, rpmVerifyAttrs omitMask)
@@ -2365,7 +2406,7 @@ int rpmfiArchiveReadToFilePsm(rpmfi fi, FD_t fd, int nodigest, rpmpsm psm)
 
     rpm_loff_t left = rpmfiFSize(fi);
     const unsigned char * fidigest = NULL;
-    pgpHashAlgo digestalgo = 0;
+    rpmHashAlgo digestalgo = 0;
     int rc = 0;
     char buf[BUFSIZ*4];
 
@@ -2403,7 +2444,7 @@ int rpmfiArchiveReadToFilePsm(rpmfi fi, FD_t fd, int nodigest, rpmpsm psm)
 		rc = RPMERR_DIGEST_MISMATCH;
 
 		/* ...but in old packages, empty files have zeros for digest */
-		if (rpmfiFSize(fi) == 0 && digestalgo == PGPHASHALGO_MD5) {
+		if (rpmfiFSize(fi) == 0 && digestalgo == RPM_HASH_MD5) {
 		    uint8_t zeros[diglen];
 		    memset(&zeros, 0, diglen);
 		    if (memcmp(zeros, fidigest, diglen) == 0)
@@ -2466,6 +2507,8 @@ char * rpmfileStrerror(int rc)
     case RPMERR_DIGEST_MISMATCH: s = _("Digest mismatch");	break;
     case RPMERR_INTERNAL:	s = _("Internal error");	break;
     case RPMERR_UNMAPPED_FILE:	s = _("Archive file not in header"); break;
+    case RPMERR_INVALID_SYMLINK: s = _("Invalid symlink");	break;
+    case RPMERR_ENOTDIR:	s = strerror(ENOTDIR);	break;
     case RPMERR_ENOENT:	s = strerror(ENOENT); break;
     case RPMERR_ENOTEMPTY:	s = strerror(ENOTEMPTY); break;
     case RPMERR_EXIST_AS_DIR:
