@@ -355,7 +355,6 @@ static int tryReadHeader(rpmts ts, struct rpmEIU * eiu, Header * hdrp)
 	   fd = NULL;
        }
        eiu->numFailed++;
-       *eiu->fnp = NULL;
        return RPMRC_FAIL;
    }
 
@@ -371,7 +370,6 @@ static int tryReadHeader(rpmts ts, struct rpmEIU * eiu, Header * hdrp)
    if (eiu->rpmrc == RPMRC_FAIL) {
        rpmlog(RPMLOG_ERR, _("%s cannot be installed\n"), *eiu->fnp);
        eiu->numFailed++;
-       *eiu->fnp = NULL;
    }
 
    return RPMRC_OK;
@@ -524,7 +522,10 @@ restart:
 			_("skipping %s - transfer failed\n"), fileURL);
 		eiu->numFailed++;
 		eiu->pkgURL[eiu->pkgx] = NULL;
-		tfn = _free(tfn);
+		if (tfn) {
+		    (void) unlink(tfn);
+		    tfn = _free(tfn);
+		}
 		break;
 	    }
 	    eiu->pkgState[eiu->pkgx] = 1;
@@ -556,20 +557,29 @@ restart:
 	rpmlog(RPMLOG_DEBUG, "============== %s\n", *eiu->fnp);
 	(void) urlPath(*eiu->fnp, &fileName);
 
-	if (tryReadHeader(ts, eiu, &h) == RPMRC_FAIL)
+	if (tryReadHeader(ts, eiu, &h) == RPMRC_FAIL) {
+	    if (eiu->pkgState[eiu->fnp - eiu->pkgURL] == 1)
+		(void) unlink(*eiu->fnp);
+	    *eiu->fnp = _free(*eiu->fnp);
 	    continue;
+	}
 
 	if (eiu->rpmrc == RPMRC_NOTFOUND) {
 	    rc = tryReadManifest(eiu);
 	    if (rc == RPMRC_OK) {
 	        eiu->prevx++;
+		headerFree(h);
 	        goto restart;
 	    }
+	} else if (eiu->rpmrc == RPMRC_FAIL) {
+	    if (eiu->pkgState[eiu->fnp - eiu->pkgURL] == 1)
+		(void) unlink(*eiu->fnp);
+	    *eiu->fnp = _free(*eiu->fnp);
 	}
 
 	if (headerIsSource(h)) {
+	    headerFree(h);
 	    if (ia->installInterfaceFlags & INSTALL_FRESHEN) {
-		headerFree(h);
 	        continue;
 	    }
 	    rpmlog(RPMLOG_DEBUG, "\tadded source package [%d]\n",
@@ -594,6 +604,7 @@ restart:
 		rpmlog(RPMLOG_ERR, _("package %s is not relocatable\n"),
 		       headerGetString(h, RPMTAG_NAME));
 		eiu->numFailed++;
+		headerFree(h);
 		goto exit;
 	    }
 	}
@@ -672,8 +683,17 @@ exit:
 	    eiu->pkgURL[i] = _free(eiu->pkgURL[i]);
 	}
     }
+    if (eiu->sourceURL != NULL) {
+        for (i = 0; i < eiu->numSRPMS; i++) {
+	    if (eiu->sourceURL[i] == NULL) continue;
+	    if (eiu->pkgState[i] == 1)
+	        (void) unlink(eiu->sourceURL[i]);
+	    eiu->sourceURL[i] = _free(eiu->sourceURL[i]);
+	}
+    }
     eiu->pkgState = _free(eiu->pkgState);
     eiu->pkgURL = _free(eiu->pkgURL);
+    eiu->sourceURL = _free(eiu->sourceURL);
     eiu->argv = _free(eiu->argv);
     rc = eiu->numFailed;
     free(eiu);
