@@ -1,7 +1,6 @@
 #include "rpmsystem-py.h"
 
-#include <rpm/rpmtypes.h>
-#include <rpm/rpmpgp.h>
+#include <rpm/rpmtag.h>
 
 #include "header-py.h"
 #include "rpmfi-py.h"
@@ -338,6 +337,8 @@ static PyObject * rpmfi_new(PyTypeObject * subtype, PyObject *args, PyObject *kw
     rpmstrPool pool = NULL;
     char * kwlist[] = {"header", "tag", "flags", "pool", NULL};
 
+    DEPRECATED_METHOD("method is obsolete");
+
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&|OiO&:rpmfi_init", kwlist,
 				hdrFromPyObject, &h, &to, &flags,
 				poolFromPyObject, &pool))
