@@ -439,6 +439,13 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(RPMFILE_PUBKEY);
     REGISTER_ENUM(RPMFILE_ARTIFACT);
 
+    REGISTER_ENUM(RPMFI_FLAGS_ERASE);
+    REGISTER_ENUM(RPMFI_FLAGS_INSTALL);
+    REGISTER_ENUM(RPMFI_FLAGS_VERIFY);
+    REGISTER_ENUM(RPMFI_FLAGS_QUERY);
+    REGISTER_ENUM(RPMFI_FLAGS_FILETRIGGER);
+    REGISTER_ENUM(RPMFI_FLAGS_ONLY_FILENAMES);
+
     REGISTER_ENUM(RPMDEP_SENSE_REQUIRES);
     REGISTER_ENUM(RPMDEP_SENSE_CONFLICTS);
 
