@@ -83,6 +83,10 @@ static int process_package(rpmts ts, const char * filename)
     struct archive_entry *entry;
 
     if (!strcmp(filename, "-")) {
+        if(isatty(STDIN_FILENO)) {
+            fprintf(stderr, "Error: missing input RPM package\n");
+            exit(EXIT_FAILURE);
+        }
 	fdi = fdDup(STDIN_FILENO);
     } else {
 	fdi = Fopen(filename, "r.ufdio");
@@ -141,13 +145,13 @@ static int process_package(rpmts ts, const char * filename)
 	fprintf(stderr, "Error: Format pax restricted is not supported\n");
 	exit(EXIT_FAILURE);
     }
-    if (!strcmp(filename, "-")) {
-	if (isatty(STDOUT_FILENO)) {
+    if (!isatty(STDOUT_FILENO)) {
+	archive_write_open_fd(a, STDOUT_FILENO);
+    } else {
+        if (!strcmp(filename, "-")) {
 	    fprintf(stderr, "Error: refusing to output archive data to a terminal.\n");
 	    exit(EXIT_FAILURE);
 	}
-	archive_write_open_fd(a, STDOUT_FILENO);
-    } else {
 	char * outname;
 	if (urlIsURL(filename)) {
 	    const char * fname = strrchr(filename, '/');
@@ -233,9 +237,13 @@ int main(int argc, const char *argv[])
 
     optCon = poptGetContext(NULL, argc, argv, optionsTable, 0);
     poptSetOtherOptionHelp(optCon, "[OPTIONS]* <FILES>");
-    if (argc < 2 || poptGetNextOpt(optCon) == 0) {
-	poptPrintUsage(optCon, stderr, 0);
-	exit(EXIT_FAILURE);
+    while ((rc = poptGetNextOpt(optCon)) != -1) {
+	if (rc < 0) {
+	    fprintf(stderr, "%s: %s\n",
+		    poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
+		    poptStrerror(rc));
+	    exit(EXIT_FAILURE);
+	}
     }
 
     rpmts ts = rpmtsCreate();
