@@ -34,24 +34,33 @@ static struct rpmBuildArguments_s rpmBTArgs;
 #define	POPT_BA			0x6261
 #define	POPT_BB			0x6262
 #define	POPT_BC			0x6263
+#define	POPT_BD			0x6264
+#define	POPT_BF			0x6266
 #define	POPT_BI			0x6269
 #define	POPT_BL			0x626c
 #define	POPT_BP			0x6270
 #define	POPT_BS			0x6273
+#define	POPT_BR			0x6272
 #define	POPT_RA			0x4261
 #define	POPT_RB			0x4262
 #define	POPT_RC			0x4263
+#define	POPT_RD			0x4264
+#define	POPT_RF			0x4266
 #define	POPT_RI			0x4269
 #define	POPT_RL			0x426c
 #define	POPT_RP			0x4270
 #define	POPT_RS			0x4273
+#define	POPT_RR			0x4272
 #define	POPT_TA			0x7461
 #define	POPT_TB			0x7462
 #define	POPT_TC			0x7463
+#define	POPT_TD			0x7464
+#define	POPT_TF			0x7466
 #define	POPT_TI			0x7469
 #define	POPT_TL			0x746c
 #define	POPT_TP			0x7470
 #define	POPT_TS			0x7473
+#define	POPT_TR			0x7472
 
 extern int _fsm_debug;
 
@@ -59,7 +68,7 @@ static rpmSpecFlags spec_flags = 0;	/*!< Bit(s) to control spec parsing. */
 static int noDeps = 0;			/*!< from --nodeps */
 static int shortCircuit = 0;		/*!< from --short-circuit */
 static char buildMode = 0;		/*!< Build mode (one of "btBC") */
-static char buildChar = 0;		/*!< Build stage (one of "abcilps ") */
+static char buildChar = 0;		/*!< Build stage (one of "abcdfilps ") */
 static rpmBuildFlags nobuildAmount = 0;	/*!< Build stage disablers */
 static ARGV_t build_targets = NULL;	/*!< Target platform(s) */
 static int buildInPlace = 0;		/*!< from --build-in-place */
@@ -77,24 +86,33 @@ static void buildArgCallback( poptContext con,
     case POPT_BA:
     case POPT_BB:
     case POPT_BC:
+    case POPT_BD:
+    case POPT_BF:
     case POPT_BI:
     case POPT_BL:
     case POPT_BP:
     case POPT_BS:
+    case POPT_BR:
     case POPT_RA:
     /* case POPT_RB: same value as POPT_REBUILD */
     case POPT_RC:
+    case POPT_RD:
+    case POPT_RF:
     case POPT_RI:
     case POPT_RL:
     case POPT_RP:
     case POPT_RS:
+    case POPT_RR:
     case POPT_TA:
     case POPT_TB:
     case POPT_TC:
+    case POPT_TD:
+    case POPT_TF:
     case POPT_TI:
     case POPT_TL:
     case POPT_TP:
     case POPT_TS:
+    case POPT_TR:
 	if (opt->val == POPT_BS || opt->val == POPT_TS)
 	    noDeps = 1;
 	if (buildMode == '\0' && buildChar == '\0') {
@@ -138,11 +156,17 @@ static struct poptOption rpmBuildPoptTable[] = {
  { "bp", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BP,
 	N_("build through %prep (unpack sources and apply patches) from <specfile>"),
 	N_("<specfile>") },
+ { "bf", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BF,
+	N_("build through %conf (%prep, then %conf) from <specfile>"),
+	N_("<specfile>") },
  { "bc", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BC,
-	N_("build through %build (%prep, then compile) from <specfile>"),
+	N_("build through %build (%prep, %conf, then compile) from <specfile>"),
+	N_("<specfile>") },
+ { "bd", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BD,
+	N_("check build dependencies <specfile>"),
 	N_("<specfile>") },
  { "bi", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BI,
-	N_("build through %install (%prep, %build, then install) from <specfile>"),
+	N_("build through %install (%prep, %conf, %build, then install) from <specfile>"),
 	N_("<specfile>") },
  { "bl", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BL,
 	N_("verify %files section from <specfile>"),
@@ -156,15 +180,24 @@ static struct poptOption rpmBuildPoptTable[] = {
  { "bs", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BS,
 	N_("build source package only from <specfile>"),
 	N_("<specfile>") },
+ { "br", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_BR,
+	N_("build source package only from <specfile> - calculate dynamic build requires"),
+	N_("<specfile>") },
 
  { "rp", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RP,
 	N_("build through %prep (unpack sources and apply patches) from <source package>"),
 	N_("<source package>") },
+ { "rf", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RF,
+	N_("build through %conf (%prep, then %conf) from <specfile>"),
+	N_("<specfile>") },
  { "rc", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RC,
-	N_("build through %build (%prep, then compile) from <source package>"),
+	N_("build through %build (%prep, %conf, then compile) from <source package>"),
+	N_("<source package>") },
+ { "rd", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RD,
+	N_("check build dependencies <source package>"),
 	N_("<source package>") },
  { "ri", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RI,
-	N_("build through %install (%prep, %build, then install) from <source package>"),
+	N_("build through %install (%prep, %conf, %build, then install) from <source package>"),
 	N_("<source package>") },
  { "rl", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RL,
 	N_("verify %files section from <source package>"),
@@ -178,15 +211,24 @@ static struct poptOption rpmBuildPoptTable[] = {
  { "rs", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RS,
 	N_("build source package only from <source package>"),
 	N_("<source package>") },
+ { "rr", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_RR,
+	N_("build source package only from <source package> - calculate dynamic build requires"),
+	N_("<source package>") },
 
  { "tp", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TP,
 	N_("build through %prep (unpack sources and apply patches) from <tarball>"),
 	N_("<tarball>") },
+ { "tf", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TF,
+	N_("build through %conf (%prep, then %conf) from <specfile>"),
+	N_("<specfile>") },
  { "tc", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TC,
-	N_("build through %build (%prep, then compile) from <tarball>"),
+	N_("build through %build (%prep, %conf, then compile) from <tarball>"),
+	N_("<tarball>") },
+ { "td", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TD,
+	N_("check build dependencies <tarball>"),
 	N_("<tarball>") },
  { "ti", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TI,
-	N_("build through %install (%prep, %build, then install) from <tarball>"),
+	N_("build through %install (%prep, %conf, %build, then install) from <tarball>"),
 	N_("<tarball>") },
  { "tl", 0, POPT_ARGFLAG_ONEDASH|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_TL,
 	N_("verify %files section from <tarball>"),
@@ -200,7 +242,9 @@ static struct poptOption rpmBuildPoptTable[] = {
  { "ts", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TS,
 	N_("build source package only from <tarball>"),
 	N_("<tarball>") },
-
+ { "tr", 0, POPT_ARGFLAG_ONEDASH, 0, POPT_TR,
+	N_("build source package only from <tarball> - calculate dynamic build requires"),
+	N_("<tarball>") },
  { "rebuild", '\0', 0, 0, POPT_REBUILD,
 	N_("build binary package from <source package>"),
 	N_("<source package>") },
@@ -226,7 +270,7 @@ static struct poptOption rpmBuildPoptTable[] = {
 	N_("generate package header(s) compatible with (legacy) rpm v3 packaging"),
 	NULL},
 
- { "noclean", '\0', POPT_BIT_SET, &nobuildAmount, RPMBUILD_CLEAN,
+ { "noclean", '\0', POPT_BIT_SET, &nobuildAmount, RPMBUILD_CLEAN|RPMBUILD_RMBUILD,
 	N_("do not execute %clean stage of the build"), NULL },
  { "noprep", '\0', POPT_BIT_SET, &nobuildAmount, RPMBUILD_PREP,
 	N_("do not execute %prep stage of the build"), NULL },
@@ -273,20 +317,6 @@ static struct poptOption optionsTable[] = {
    POPT_TABLEEND
 };
 
-static int checkSpec(rpmts ts, rpmSpec spec)
-{
-    int rc;
-    rpmps ps = rpmSpecCheckDeps(ts, spec);
-
-    if (ps) {
-	rpmlog(RPMLOG_ERR, _("Failed build dependencies:\n"));
-	rpmpsPrint(NULL, ps);
-    }
-    rc = (ps != NULL);
-    rpmpsFree(ps);
-    return rc;
-}
-
 static int isSpecFile(const char * specfile)
 {
     char buf[256];
@@ -337,18 +367,16 @@ static int isSpecFile(const char * specfile)
 static char * getTarSpec(const char *arg)
 {
     char *specFile = NULL;
-    char *specDir;
-    char *specBase;
-    char *tmpSpecFile;
+    char *specFinal = NULL;
     const char **spec;
     char tarbuf[BUFSIZ];
-    int gotspec = 0, res;
+    int gotspec = 0;
+    FD_t fd = NULL;
     static const char *tryspec[] = { "Specfile", "\\*.spec", NULL };
 
-    specDir = rpmGetPath("%{_specdir}", NULL);
-    tmpSpecFile = rpmGetPath("%{_specdir}/", "rpm-spec.XXXXXX", NULL);
-
-    (void) close(mkstemp(tmpSpecFile));
+    specFile = rpmGetPath("%{_specdir}/", "rpm-spec.XXXXXX", NULL);
+    if (!(fd = rpmMkTemp(specFile)))
+	goto exit;
 
     for (spec = tryspec; *spec != NULL; spec++) {
 	FILE *fp;
@@ -357,7 +385,7 @@ static char * getTarSpec(const char *arg)
 
 	cmd = rpmExpand("%{uncompress: ", arg, "} | ",
 			"%{__tar} xOvof - --wildcards ", *spec,
-			" 2>&1 > ", tmpSpecFile, NULL);
+			" 2>&1 > ", specFile, NULL);
 
 	if (!(fp = popen(cmd, "r"))) {
 	    rpmlog(RPMLOG_ERR, _("Failed to open tar pipe: %m\n"));
@@ -373,47 +401,36 @@ static char * getTarSpec(const char *arg)
 		specfiles++;
 	    }
 	    pclose(fp);
-	    gotspec = (specfiles == 1) && isSpecFile(tmpSpecFile);
+	    gotspec = (specfiles == 1) && isSpecFile(specFile);
 	    if (specfiles > 1) {
 		rpmlog(RPMLOG_ERR, _("Found more than one spec file in %s\n"), arg);
 		goto exit;
 	    }
 	}
 
-	if (!gotspec) 
-	    unlink(tmpSpecFile);
+	if (!gotspec)
+	    unlink(specFile);
 	free(cmd);
     }
 
     if (!gotspec) {
     	rpmlog(RPMLOG_ERR, _("Failed to read spec file from %s\n"), arg);
-	goto exit;
-    }
-
-    specBase = basename(tarbuf);
-    /* remove trailing \n */
-    specBase[strlen(specBase)-1] = '\0';
-
-    rasprintf(&specFile, "%s/%s", specDir, specBase);
-    res = rename(tmpSpecFile, specFile);
-
-    if (res) {
-    	rpmlog(RPMLOG_ERR, _("Failed to rename %s to %s: %m\n"),
-		tmpSpecFile, specFile);
-    	free(specFile);
-	specFile = NULL;
     } else {
-    	/* mkstemp() can give unnecessarily strict permissions, fixup */
-	mode_t mask;
-	umask(mask = umask(0));
-	(void) chmod(specFile, 0666 & ~mask);
+	/* remove trailing \n */
+	tarbuf[strlen(tarbuf)-1] = '\0';
+	specFinal = rpmExpand("%{_specdir}/%{basename:", tarbuf, "}", NULL);
+	if (rename(specFile, specFinal)) {
+	    rpmlog(RPMLOG_ERR, _("Failed to rename %s to %s: %m\n"),
+		    specFile, specFinal);
+	    unlink(specFile);
+	    specFinal = _free(specFinal);
+	}
     }
 
 exit:
-    (void) unlink(tmpSpecFile);
-    free(tmpSpecFile);
-    free(specDir);
-    return specFile;
+    free(specFile);
+    Fclose(fd);
+    return specFinal;
 }
 
 static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
@@ -423,7 +440,6 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     char * specFile = NULL;
     rpmSpec spec = NULL;
     int rc = 1; /* assume failure */
-    int justRm = ((buildAmount & ~(RPMBUILD_RMSOURCE|RPMBUILD_RMSPEC)) == 0);
     rpmSpecFlags specFlags = spec_flags;
 
     /* Override default BUILD value for _builddir */
@@ -436,15 +452,13 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     if (ba->buildRootOverride)
 	buildRootURL = rpmGenPath(NULL, ba->buildRootOverride, NULL);
 
-    /* Create build tree if necessary */
-    const char * buildtree = "%{_topdir}:%{_specdir}:%{_sourcedir}:%{_builddir}:%{_rpmdir}:%{_srcrpmdir}:%{_buildrootdir}";
-    const char * rootdir = rpmtsRootDir(ts);
-    if (rpmMkdirs(!rstreq(rootdir, "/") ? rootdir : NULL , buildtree)) {
-	goto exit;
-    }
+    const char *rootdir = rpmtsRootDir(ts);
+    const char *root = !rstreq(rootdir, "/") ? rootdir : NULL;
 
     if (buildMode == 't') {
     	char *srcdir = NULL, *dir;
+	if (rpmMkdirs(root, "%{_specdir}"))
+	    goto exit;
 
 	specFile = getTarSpec(arg);
 	if (!specFile)
@@ -499,7 +513,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 
     /* Parse the spec file */
 #define	_anyarch(_f)	\
-(((_f)&(RPMBUILD_PREP|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
+(((_f)&(RPMBUILD_PREP|RPMBUILD_CONF|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
     if (_anyarch(buildAmount))
 	specFlags |= RPMSPEC_ANYARCH;
 #undef	_anyarch
@@ -509,12 +523,11 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	goto exit;
     }
 
-    /* Check build prerequisites if necessary, unless disabled */
-    if (!justRm && !noDeps && checkSpec(ts, spec)) {
+    /* Create build tree if necessary */
+    if (rpmMkdirs(root, "%{_topdir}:%{_builddir}:%{_rpmdir}:%{_srcrpmdir}:%{_buildrootdir}"))
 	goto exit;
-    }
 
-    if (rpmSpecBuild(spec, ba)) {
+    if ((rc = rpmSpecBuild(ts, spec, ba))) {
 	goto exit;
     }
     
@@ -593,8 +606,6 @@ int main(int argc, char *argv[])
 
     poptContext optCon = NULL;
 
-    xsetprogname(argv[0]); /* Portability call -- see system.h */
-
     optCon = rpmcliInit(argc, argv, optionsTable);
 
     /* Args required only when building, let lone --eval etc through */
@@ -628,29 +639,60 @@ int main(int argc, char *argv[])
     (void) rpmtsSetRootDir(ts, rpmcliRootDir);
     rpmtsSetFlags(ts, rpmtsFlags(ts) | RPMTRANS_FLAG_NOPLUGINS);
 
+    /* Mind the fallthrough order - it's the reverse of the build process */
     switch (buildChar) {
     case 'a':
 	ba->buildAmount |= RPMBUILD_PACKAGESOURCE;
+	/* fallthrough */
     case 'b':
 	ba->buildAmount |= RPMBUILD_PACKAGEBINARY;
 	ba->buildAmount |= RPMBUILD_CLEAN;
 	if ((buildChar == 'b') && shortCircuit)
 	    break;
+	ba->buildAmount |= RPMBUILD_RMBUILD;
+	/* fallthrough */
     case 'i':
 	ba->buildAmount |= RPMBUILD_INSTALL;
 	ba->buildAmount |= RPMBUILD_CHECK;
 	if ((buildChar == 'i') && shortCircuit)
 	    break;
+	/* fallthrough */
     case 'c':
 	ba->buildAmount |= RPMBUILD_BUILD;
 	if ((buildChar == 'c') && shortCircuit)
 	    break;
+	/* fallthrough */
+    case 'f':
+	ba->buildAmount |= RPMBUILD_CONF;
+	ba->buildAmount |= RPMBUILD_BUILDREQUIRES;
+	if (!noDeps) {
+	    ba->buildAmount |= RPMBUILD_DUMPBUILDREQUIRES;
+	    ba->buildAmount |= RPMBUILD_CHECKBUILDREQUIRES;
+	}
+	if ((buildChar == 'f') && shortCircuit)
+	    break;
+	/* fallthrough */
     case 'p':
 	ba->buildAmount |= RPMBUILD_PREP;
+	if (!noDeps) {
+	    ba->buildAmount |= RPMBUILD_CHECKBUILDREQUIRES;
+	}
 	break;
     case 'l':
 	ba->buildAmount |= RPMBUILD_FILECHECK;
 	break;
+    case 'r':
+	/* fallthrough */
+    case 'd':
+	if (!shortCircuit)
+	    ba->buildAmount |= RPMBUILD_PREP;
+	ba->buildAmount |= RPMBUILD_BUILDREQUIRES;
+	ba->buildAmount |= RPMBUILD_DUMPBUILDREQUIRES;
+	if (!noDeps)
+	    ba->buildAmount |= RPMBUILD_CHECKBUILDREQUIRES;
+	if (buildChar == 'd')
+	    break;
+	/* fallthrough */
     case 's':
 	ba->buildAmount |= RPMBUILD_PACKAGESOURCE;
 	break;
@@ -662,6 +704,7 @@ int main(int argc, char *argv[])
     case MODE_RECOMPILE:
 	if (bigMode == MODE_REBUILD &&
 	    buildChar != 'p' &&
+	    buildChar != 'f' &&
 	    buildChar != 'c' &&
 	    buildChar != 'i' &&
 	    buildChar != 'l') {
