@@ -58,8 +58,8 @@ struct rpmMacroEntry_s {
 struct rpmMacroContext_s {
     rpmMacroEntry *tab;  /*!< Macro entry table (array of pointers). */
     int n;      /*!< No. of macros. */
-    int depth;		 /*!< Depth tracking when recursing from Lua  */
-    int level;		 /*!< Scope level tracking when recursing from Lua  */
+    int depth;		 /*!< Depth tracking on external recursion */
+    int level;		 /*!< Scope level tracking when on external recursion */
     pthread_mutex_t lock;
     pthread_mutexattr_t lockattr;
 };
@@ -586,7 +586,16 @@ static void doExpressionExpansion(MacroBuf mb, const char * expr, size_t len)
 {
     char *buf = rstrndup(expr, len);
     char *result;
+    rpmMacroContext mc = mb->mc;
+    int odepth = mc->depth;
+    int olevel = mc->level;
+
+    mc->depth = mb->depth;
+    mc->level = mb->level;
     result = rpmExprStrFlags(buf, RPMEXPR_EXPAND);
+    mc->depth = odepth;
+    mc->level = olevel;
+
     if (!result) {
 	mb->error = 1;
 	goto exit;
@@ -1223,6 +1232,11 @@ static void doTrace(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
     }
 }
 
+static void doRpmver(MacroBuf mb, rpmMacroEntry me, ARGV_t argv, size_t *parsed)
+{
+    mbAppendStr(mb, VERSION);
+}
+
 static struct builtins_s {
     const char * name;
     macroFunc func;
@@ -1249,6 +1263,7 @@ static struct builtins_s {
     { "lua",		doLua,		1,	ME_PARSE },
     { "macrobody",	doBody,		1,	0 },
     { "quote",		doFoo,		1,	0 },
+    { "rpmversion",	doRpmver,	0,	0 },
     { "shrink",		doFoo,		1,	0 },
     { "suffix",		doFoo,		1,	0 },
     { "trace",		doTrace,	0,	0 },
