@@ -1058,6 +1058,8 @@ static rpmzstd rpmzstdNew(int fdno, const char *fmode)
     char *te = t + sizeof(stdio) - 2;
     int c;
     int threads = 0;
+    int windowlog = 27;
+    int longdist = 0;
 
     switch ((c = *s++)) {
     case 'a':
@@ -1087,23 +1089,39 @@ static rpmzstd rpmzstdNew(int fdno, const char *fmode)
 	    flags |= O_RDWR;
 	    continue;
 	case 'T':
-	    if (*s >= '0' && *s <= '9') {
+	    if (*s >= '0' && *s <= '9')
 		threads = strtol(s, (char **)&s, 10);
-		/* T0 means automatic detection */
-		if (threads == 0)
-		    threads = -1;
-	    }
+	    /* T0 means automatic detection */
+	    if (threads == 0)
+		threads = -1;
+	    continue;
+    case 'L':
+	    c = *s++;
+	    longdist = 1;
+	    if (c >= (int)'0' && c <= (int)'9') {
+		windowlog = strtol(s-1, (char **)&s, 10);
+		ZSTD_bounds bounds = ZSTD_cParam_getBounds(ZSTD_c_windowLog);
+		if (windowlog < bounds.lowerBound){
+		    windowlog = bounds.lowerBound;
+		    rpmlog(RPMLOG_WARNING, "Invalid long distance matching window log for zstd. Using %i instead.\n", bounds.lowerBound);
+		}
+		if (windowlog > bounds.upperBound) {
+		    windowlog = bounds.upperBound;
+		    rpmlog(RPMLOG_WARNING, "Invalid long distance matching window log for zstd. Using %i instead.\n", bounds.upperBound);
+		}
+   	    }
 	    continue;
 	default:
 	    if (c >= (int)'0' && c <= (int)'9') {
 		level = strtol(s-1, (char **)&s, 10);
-		if (level < 1){
-		    level = 1;
-		    rpmlog(RPMLOG_WARNING, "Invalid compression level for zstd. Using %i instead.\n", 1);
+		ZSTD_bounds bounds = ZSTD_cParam_getBounds(ZSTD_c_compressionLevel);
+		if (level < bounds.lowerBound){
+		    level = bounds.lowerBound;
+		    rpmlog(RPMLOG_WARNING, "Invalid compression level for zstd. Using %i instead.\n", bounds.lowerBound);
 		}
-		if (level > 19) {
-		    level = 19;
-		    rpmlog(RPMLOG_WARNING, "Invalid compression level for zstd. Using %i instead.\n", 19);
+		if (level > bounds.upperBound) {
+		    level = bounds.upperBound;
+		    rpmlog(RPMLOG_WARNING, "Invalid compression level for zstd. Using %i instead.\n", bounds.upperBound);
 		}
 	    }
 	    continue;
@@ -1131,6 +1149,13 @@ static rpmzstd rpmzstdNew(int fdno, const char *fmode)
 	    goto err;
 	}
 
+	if (longdist) {
+	    if (ZSTD_isError(ZSTD_CCtx_setParameter(_stream, ZSTD_c_enableLongDistanceMatching, longdist))
+	     || ZSTD_isError(ZSTD_CCtx_setParameter(_stream, ZSTD_c_windowLog, windowlog))) {
+	        goto err;
+	    }
+	}
+
 	threads = get_compression_threads(threads);
 	if (threads > 0) {
 	    if (ZSTD_isError (ZSTD_CCtx_setParameter(_stream, ZSTD_c_nbWorkers, threads)))
@@ -1418,7 +1443,7 @@ int Fclose(FD_t fd)
 	}
 
 	/* Debugging stats for compresed types */
-	if ((_rpmio_debug || rpmIsDebug()) && fps->fdno == -1)
+	if (_rpmio_debug && fps->fdno == -1)
 	    fdstat_print(fd, fps->io->ioname, stderr);
 
 	/* Leave freeing the last one after stats */
