@@ -9,6 +9,7 @@
 #include <errno.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmmacro.h>
+#include <rpm/rpmstring.h>
 #include "debug.h"
 
 typedef struct rpmlogCtx_s * rpmlogCtx;
@@ -16,6 +17,7 @@ struct rpmlogCtx_s {
     pthread_rwlock_t lock;
     unsigned mask;
     int nrecs;
+    int nrecsPri[RPMLOG_NPRIS];
     rpmlogRec recs;
     rpmlogCallback cbfunc;
     rpmlogCallbackData cbdata;
@@ -33,7 +35,7 @@ static rpmlogCtx rpmlogCtxAcquire(int write)
 {
     static struct rpmlogCtx_s _globalCtx = { PTHREAD_RWLOCK_INITIALIZER,
 					     RPMLOG_UPTO(RPMLOG_NOTICE),
-					     0, NULL, NULL, NULL, NULL };
+					     0, {0}, NULL, NULL, NULL, NULL };
     rpmlogCtx ctx = &_globalCtx;
     int xx;
 
@@ -54,16 +56,30 @@ static rpmlogCtx rpmlogCtxRelease(rpmlogCtx ctx)
     return NULL;
 }
 
-int rpmlogGetNrecs(void)
+int rpmlogGetNrecsByMask(unsigned mask)
 {
     rpmlogCtx ctx = rpmlogCtxAcquire(0);
-    int nrecs = -1;
-    if (ctx)
+    int nrecs = 0;
+
+    if (ctx == NULL)
+	return -1;
+
+    if (mask) {
+	for (int i = 0; i < RPMLOG_NPRIS; i++, mask >>= 1)
+	    if (mask & 1)
+	        nrecs += ctx->nrecsPri[i];
+    } else
 	nrecs = ctx->nrecs;
+
     rpmlogCtxRelease(ctx);
     return nrecs;
 }
 
+int rpmlogGetNrecs(void)
+{
+    return rpmlogGetNrecsByMask(0);
+}
+
 int rpmlogCode(void)
 {
     int code = -1;
@@ -98,7 +114,7 @@ rpmlogLvl rpmlogRecPriority(rpmlogRec rec)
     return (rec != NULL) ? rec->pri : (rpmlogLvl)-1;
 }
 
-void rpmlogPrint(FILE *f)
+void rpmlogPrintByMask(FILE *f, unsigned mask)
 {
     rpmlogCtx ctx = rpmlogCtxAcquire(0);
 
@@ -110,6 +126,8 @@ void rpmlogPrint(FILE *f)
 
     for (int i = 0; i < ctx->nrecs; i++) {
 	rpmlogRec rec = ctx->recs + i;
+	if (mask && ((RPMLOG_MASK(rec->pri) & mask) == 0))
+	    continue;
 	if (rec->message && *rec->message)
 	    fprintf(f, "    %s", rec->message);
     }
@@ -117,6 +135,11 @@ void rpmlogPrint(FILE *f)
     rpmlogCtxRelease(ctx);
 }
 
+void rpmlogPrint(FILE *f)
+{
+    rpmlogPrintByMask(f, 0);
+}
+
 void rpmlogClose (void)
 {
     rpmlogCtx ctx = rpmlogCtxAcquire(1);
@@ -130,6 +153,7 @@ void rpmlogClose (void)
     }
     ctx->recs = _free(ctx->recs);
     ctx->nrecs = 0;
+    memset(ctx->nrecsPri, 0, sizeof(ctx->nrecsPri));
 
     rpmlogCtxRelease(ctx);
 }
@@ -378,6 +402,7 @@ static void dolog(struct rpmlogRec_s *rec, int saverec)
 	ctx->recs[ctx->nrecs+1].code = 0;
 	ctx->recs[ctx->nrecs+1].message = NULL;
 	ctx->nrecs++;
+	ctx->nrecsPri[rec->pri]++;
     }
     cbfunc = ctx->cbfunc;
     cbdata = ctx->cbdata;
