@@ -3,8 +3,7 @@
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
-#include <lposix.h>
-#include <lrexlib.h>
+#include "lposix.h"
 
 #ifndef LUA_LOADED_TABLE
 /* feature introduced in Lua 5.3.4 */
@@ -45,13 +44,13 @@ struct rpmlua_s {
     rpmluapb printbuf;
 };
 
-#define INITSTATE(_lua, lua) \
-    rpmlua lua = _lua ? _lua : \
+#define INITSTATE(lua) \
+    (lua = lua ? lua : \
 	    (globalLuaState ? globalLuaState : \
 			\
 			(globalLuaState = rpmluaNew()) \
 			\
-	    )
+	    ))
 
 struct rpmluapb_s {
     size_t alloced;
@@ -62,38 +61,32 @@ struct rpmluapb_s {
 
 static rpmlua globalLuaState = NULL;
 
-static char *(*nextFileFunc)(void *) = NULL;
-static void *nextFileFuncParam = NULL;
-
 static int luaopen_rpm(lua_State *L);
 static int rpm_print(lua_State *L);
 static int rpm_exit(lua_State *L);
 static int rpm_redirect2null(lua_State *L);
 
+static int rpmluaHookPushArg(lua_State *L, int argt, rpmhookArgv *arg);
+static int rpmluaHookGetArg(lua_State *L, int idx, rpmhookArgv *arg);
+
 static int pusherror(lua_State *L, int code, const char *info)
 {
     lua_pushnil(L);
-    if (info == NULL)
-	lua_pushstring(L, strerror(code));
-    else
-	lua_pushfstring(L, "%s: %s", info, strerror(code));
+    lua_pushstring(L, info ? info : strerror(code));
     lua_pushnumber(L, code);
     return 3;
 }
 
-static int pushresult(lua_State *L, int result, const char *info)
+static int pushresult(lua_State *L, int result)
 {
-    if (result == 0) {
-	lua_pushnumber(L, result);
-	return 1;
-    }
-
-    return pusherror(L, result, info);
+    lua_pushnumber(L, result);
+    return 1;
 }
 
 rpmlua rpmluaGetGlobalState(void)
 {
-    INITSTATE(NULL, lua);
+    rpmlua lua = NULL;
+    INITSTATE(lua);
     return lua;
 }
 
@@ -118,7 +111,6 @@ rpmlua rpmluaNew()
 
     static const luaL_Reg extlibs[] = {
 	{"posix", luaopen_posix},
-	{"rex", luaopen_rex},
 	{"rpm", luaopen_rpm},
 	{NULL, NULL},
     };
@@ -133,21 +125,27 @@ rpmlua rpmluaNew()
 
     for (lib = extlibs; lib->name; lib++) {
 	luaL_requiref(L, lib->name, lib->func, 1);
+	lua_pop(L, 1);
     }
     lua_pushcfunction(L, rpm_print);
     lua_setglobal(L, "print");
 
     lua_getglobal(L, "os");
     luaL_setfuncs(L, os_overrides, 0);
+    lua_pop(L, 1);
+
     lua_getglobal(L, "posix");
     luaL_setfuncs(L, posix_overrides, 0);
+    lua_pop(L, 1);
 
     lua_getglobal(L, "package");
     lua_pushfstring(L, "%s/%s", rpmConfigDir(), "/lua/?.lua");
     lua_setfield(L, -2, "path");
     lua_pop(L, 1);
 
-    rpmluaSetData(lua, "lua", lua);
+    lua_pushliteral(L, "rpm_lua");
+    lua_pushlightuserdata(L, (void *)lua);
+    lua_rawset(L, LUA_REGISTRYINDEX);
 
     initlua = rpmGenPath(rpmConfigDir(), "init.lua", NULL);
     if (stat(initlua, &st) != -1)
@@ -167,57 +165,26 @@ rpmlua rpmluaFree(rpmlua lua)
     return NULL;
 }
 
-void * rpmluaGetLua(rpmlua _lua)
+static rpmlua getlua(lua_State *L)
 {
-    INITSTATE(_lua, lua);
+    rpmlua lua = NULL;
+    lua_pushliteral(L, "rpm_lua");
+    lua_rawget(L, LUA_REGISTRYINDEX);
+    if (lua_islightuserdata(L, -1))
+       lua = (rpmlua)lua_touserdata(L, -1);
+    lua_pop(L, 1);
+    return lua;
+}
+
+void * rpmluaGetLua(rpmlua lua)
+{
+    INITSTATE(lua);
     return lua->L;
 }
 
-void rpmluaRegister(rpmlua lua, const void *regfuncs, const char *lib)
+void rpmluaPushPrintBuffer(rpmlua lua)
 {
-    const luaL_Reg *funcs = regfuncs;
-    lua_getfield(lua->L, LUA_REGISTRYINDEX, LUA_LOADED_TABLE);
-    lua_getfield(lua->L, -1, lib);
-    luaL_setfuncs(lua->L, funcs, 0);
-    lua_pop(lua->L, 2);
-}
-
-void rpmluaSetData(rpmlua _lua, const char *key, const void *data)
-{
-    INITSTATE(_lua, lua);
-    lua_State *L = lua->L;
-    lua_pushliteral(L, "rpm_");
-    lua_pushstring(L, key);
-    lua_concat(L, 2);
-    if (data == NULL)
-	lua_pushnil(L);
-    else
-	lua_pushlightuserdata(L, (void *)data);
-    lua_rawset(L, LUA_REGISTRYINDEX);
-}
-
-static void *getdata(lua_State *L, const char *key)
-{
-    void *ret = NULL;
-    lua_pushliteral(L, "rpm_");
-    lua_pushstring(L, key);
-    lua_concat(L, 2);
-    lua_rawget(L, LUA_REGISTRYINDEX);
-    if (lua_islightuserdata(L, -1))
-	ret = lua_touserdata(L, -1);
-    lua_pop(L, 1);
-    return ret;
-}
-
-void *rpmluaGetData(rpmlua _lua, const char *key)
-{
-    INITSTATE(_lua, lua);
-    return getdata(lua->L, key);
-}
-
-void rpmluaPushPrintBuffer(rpmlua _lua)
-{
-    INITSTATE(_lua, lua);
+    INITSTATE(lua);
     rpmluapb prbuf = xcalloc(1, sizeof(*prbuf));
     prbuf->buf = NULL;
     prbuf->alloced = 0;
@@ -227,9 +194,9 @@ void rpmluaPushPrintBuffer(rpmlua _lua)
     lua->printbuf = prbuf;
 }
 
-char *rpmluaPopPrintBuffer(rpmlua _lua)
+char *rpmluaPopPrintBuffer(rpmlua lua)
 {
-    INITSTATE(_lua, lua);
+    INITSTATE(lua);
     rpmluapb prbuf = lua->printbuf;
     char *ret = NULL;
 
@@ -242,15 +209,9 @@ char *rpmluaPopPrintBuffer(rpmlua _lua)
     return ret;
 }
 
-void rpmluaSetNextFileFunc(char *(*func)(void *), void *funcParam)
+int rpmluaCheckScript(rpmlua lua, const char *script, const char *name)
 {
-    nextFileFunc = func;
-    nextFileFuncParam = funcParam;
-}
-
-int rpmluaCheckScript(rpmlua _lua, const char *script, const char *name)
-{
-    INITSTATE(_lua, lua);
+    INITSTATE(lua);
     lua_State *L = lua->L;
     int ret = 0;
     if (name == NULL)
@@ -275,15 +236,34 @@ static int luaopt(int c, const char *oarg, int oint, void *data)
     return 0;
 }
 
-int rpmluaRunScript(rpmlua _lua, const char *script, const char *name,
+static int rpm_pcall(lua_State *L, int nargs, int nresults, int errfunc)
+{
+    pid_t pid = getpid();
+    int status;
+
+    int rc = lua_pcall(L, nargs, nresults, errfunc);
+
+    /* Terminate unhandled fork from Lua script */
+    if (pid != getpid())
+	_exit(1);
+
+    if (waitpid(0, &status, WNOHANG) == 0) {
+	rpmlog(RPMLOG_WARNING,
+		_("runaway fork() in Lua script\n"));
+    }
+
+    return rc;
+}
+
+int rpmluaRunScript(rpmlua lua, const char *script, const char *name,
 		    const char *opts, ARGV_t args)
 {
-    INITSTATE(_lua, lua);
+    INITSTATE(lua);
     lua_State *L = lua->L;
     int ret = -1;
     int oind = 0;
-    static const char *lualocal =
-	"local opt = select(1, ...); local arg = select(2, ...);";
+    static const char *lualocal = "local opt, arg = ...;";
+    int otop = lua_gettop(L); /* this can recurse through macros */
 
     if (name == NULL)
 	name = "<lua>";
@@ -322,12 +302,25 @@ int rpmluaRunScript(rpmlua _lua, const char *script, const char *name,
 	}
     }
 
-    if (lua_pcall(L, 2, 0, 0) != 0) {
+    if (rpm_pcall(L, 2, LUA_MULTRET, 0) != 0) {
 	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	goto exit;
     }
+
+    int nret = lua_gettop(L) - otop;
+    if (nret > 0 && lua->printbuf) {
+	lua_getglobal(L, "print");
+	lua_insert(L, -(nret + 1));
+	if (rpm_pcall(L, nret, 0, 0) != 0) {
+	    rpmlog(RPMLOG_ERR, _("result print failed: %s\n"),
+		    lua_tostring(L, -1));
+	    lua_pop(L, 1);
+	    goto exit;
+	}
+    }
+
     ret = 0;
 
 exit:
@@ -335,9 +328,9 @@ exit:
     return ret;
 }
 
-int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
+int rpmluaRunScriptFile(rpmlua lua, const char *filename)
 {
-    INITSTATE(_lua, lua);
+    INITSTATE(lua);
     lua_State *L = lua->L;
     int ret = 0;
     if (luaL_loadfile(L, filename) != 0) {
@@ -345,7 +338,7 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
 	ret = -1;
-    } else if (lua_pcall(L, 0, 0, 0) != 0) {
+    } else if (rpm_pcall(L, 0, 0, 0) != 0) {
 	rpmlog(RPMLOG_ERR, _("lua script failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
@@ -354,72 +347,142 @@ int rpmluaRunScriptFile(rpmlua _lua, const char *filename)
     return ret;
 }
 
-/* From lua.c */
-static int rpmluaReadline(lua_State *L, const char *prompt)
+static char *lamereadline(char *prompt)
 {
-   static char buffer[1024];
-   if (prompt) {
-      (void) fputs(prompt, stdout);
-      (void) fflush(stdout);
-   }
-   if (fgets(buffer, sizeof(buffer), stdin) == NULL) {
-      return 0;  /* read fails */
-   } else {
-      lua_pushstring(L, buffer);
-      return 1;
-   }
+    static char buffer[1024];
+    if (prompt) {
+	(void) fputs(prompt, stdout);
+	(void) fflush(stdout);
+    }
+    if (fgets(buffer, sizeof(buffer), stdin) == NULL)
+	return NULL;
+
+    return xstrdup(buffer);
+}
+
+/* From lua.c */
+static int rpmluaReadline(lua_State *L, char *prompt, rpmluarl rlcb)
+{
+    char *line = rlcb(prompt);
+    if (line) {
+	lua_pushstring(L, line);
+	free(line);
+	return 1;
+    }
+    return 0;
 }
 
 /* Based on lua.c */
-static void _rpmluaInteractive(lua_State *L)
+static void _rpmluaInteractive(lua_State *L, rpmluarl rlcb)
 {
-   rpmlua lua = getdata(L, "lua");
-   (void) fputs("\n", stdout);
-   printf("RPM Interactive %s Interpreter\n", LUA_VERSION);
-   for (;;) {
-      int rc = 0;
+    if (rlcb == NULL)
+	rlcb = lamereadline;
+    rpmlua lua = getlua(L);
+    (void) fputs("\n", stdout);
+    printf("RPM Interactive %s Interpreter\n", LUA_VERSION);
+    for (;;) {
+	int rc = 0;
 
-      if (rpmluaReadline(L, "> ") == 0)
-	 break;
-      if (lua_tostring(L, -1)[0] == '=') {
-	 (void) lua_pushfstring(L, "print(%s)", lua_tostring(L, -1)+1);
-	 lua_remove(L, -2);
-      }
-      for (;;) {
-         size_t len;
-	 const char *code = lua_tolstring(L, -1, &len);
-	 rc = luaL_loadbuffer(L, code, len, "<lua>");
-	 if (rc == LUA_ERRSYNTAX &&
-	     strstr(lua_tostring(L, -1), "near `<eof>'") != NULL) {
-	    if (rpmluaReadline(L, ">> ") == 0)
-	       break;
-	    lua_remove(L, -2); /* Remove error */
-	    lua_concat(L, 2);
-	    continue;
-	 }
-	 break;
-      }
-      if (rc == 0)
-	 rc = lua_pcall(L, 0, 0, 0);
-      if (rc != 0) {
-	 fprintf(stderr, "%s\n", lua_tostring(L, -1));
-	 lua_pop(L, 1);
-      } else {
-	 char *s = rpmluaPopPrintBuffer(lua);
-	 if (s) {
-	    fprintf(stdout, "%s\n", s);
-	    free(s);
-	 }
-      }
-      lua_pop(L, 1); /* Remove line */
-   }
-   (void) fputs("\n", stdout);
+	if (rpmluaReadline(L, "> ", rlcb) == 0)
+	    break;
+	if (lua_tostring(L, -1)[0] == '=') {
+	    (void) lua_pushfstring(L, "print(%s)", lua_tostring(L, -1)+1);
+	    lua_remove(L, -2);
+	}
+	for (;;) {
+	    size_t len;
+	    const char *code = lua_tolstring(L, -1, &len);
+	    rc = luaL_loadbuffer(L, code, len, "<lua>");
+	    if (rc == LUA_ERRSYNTAX &&
+		strstr(lua_tostring(L, -1), "near `<eof>'") != NULL) {
+		if (rpmluaReadline(L, ">> ", rlcb) == 0)
+		    break;
+		lua_remove(L, -2); /* Remove error */
+		lua_concat(L, 2);
+		continue;
+	   }
+	   break;
+	}
+	if (rc == 0)
+	    rc = rpm_pcall(L, 0, 0, 0);
+	if (rc != 0) {
+	    fprintf(stderr, "%s\n", lua_tostring(L, -1));
+	    lua_pop(L, 1);
+	} else {
+	    char *s = rpmluaPopPrintBuffer(lua);
+	    if (s) {
+		fprintf(stdout, "%s\n", s);
+		free(s);
+		rpmluaPushPrintBuffer(lua);
+	    }
+	}
+	lua_pop(L, 1); /* Remove line */
+    }
+    (void) fputs("\n", stdout);
 }
 
-void rpmluaInteractive(rpmlua _lua)
+void rpmluaInteractive(rpmlua lua, rpmluarl rl)
 {
-    INITSTATE(_lua, lua);
-    _rpmluaInteractive(lua->L);
+    INITSTATE(lua);
+    _rpmluaInteractive(lua->L, rl);
+}
+
+char *rpmluaCallStringFunction(rpmlua lua, const char *function, rpmhookArgs args)
+{
+    INITSTATE(lua);
+    lua_State *L = lua->L;
+    int i;
+    char *fcall = NULL;
+    char *result = NULL;
+
+    if (!lua_checkstack(L, args->argc + 1)) {
+	rpmlog(RPMLOG_ERR, "out of lua stack space\n");
+	return NULL;
+    }
+
+    /* compile the call */
+    rasprintf(&fcall, "return (%s)(...)", function);
+    if (luaL_loadbuffer(L, fcall, strlen(fcall), function) != 0) {
+	rpmlog(RPMLOG_ERR, "%s: %s\n", function, lua_tostring(L, -1));
+	lua_pop(L, 1);
+	free(fcall);
+	return NULL;
+    }
+    free(fcall);
+
+    /* convert the arguments */
+    for (i = 0; i < args->argc; i++) {
+	if (rpmluaHookPushArg(L, args->argt[i], args->argv + i)) {
+	    rpmlog(RPMLOG_ERR, "%s: cannot convert argment type %c\n", function, args->argt[i]);
+	    lua_pop(L, i + 1);
+	    return NULL;
+	}
+    }
+
+    /* run the call */
+    if (rpm_pcall(L, args->argc, 1, 0) != 0) {
+	rpmlog(RPMLOG_ERR, "%s: %s\n", function, lua_tostring(L, -1));
+	lua_pop(L, 1);
+	return NULL;
+    }
+
+    /* convert result to a string */
+    if (lua_isnil(L, -1)) {
+	result = xstrdup("");	/* tolstring will convert to "nil" */
+    } else if (lua_isboolean(L, -1)) {
+	result = xstrdup(lua_toboolean(L, -1) ? "1" : "");
+    } else {
+	lua_getglobal(L, "tostring");
+	lua_insert(L, -2);
+	if (rpm_pcall(L, 1, 1, 0) != 0) {
+	    rpmlog(RPMLOG_ERR, "%s: %s\n", function, lua_tostring(L, -1));
+	    lua_pop(L, 1);
+	    return NULL;
+	}
+	result = xstrdup(lua_tostring(L, -1));
+    }
+    lua_pop(L, 1);
+    return result;
 }
 
 /* ------------------------------------------------------------------ */
@@ -440,7 +503,7 @@ static int rpm_vercmp(lua_State *L)
 	} else {
 	    if (v1 == NULL)
 		luaL_argerror(L, 1, "invalid version ");
-	    if (v1 == NULL)
+	    if (v2 == NULL)
 		luaL_argerror(L, 2, "invalid version ");
 	}
 	rpmverFree(v1);
@@ -531,26 +594,72 @@ static int rpm_interactive(lua_State *L)
     if (!(isatty(STDOUT_FILENO) && isatty(STDIN_FILENO)))
 	return luaL_error(L, "not a tty");
 
-    _rpmluaInteractive(L);
+    _rpmluaInteractive(L, NULL);
     return 0;
 }
 
-static int rpm_next_file(lua_State *L)
-{
-    if (nextFileFunc)
-	lua_pushstring(L, nextFileFunc(nextFileFuncParam));
-    else
-	lua_pushstring(L, NULL);
-
-    return 1;
-}
-
 typedef struct rpmluaHookData_s {
     lua_State *L;
     int funcRef;
     int dataRef;
 } * rpmluaHookData;
 
+static int rpmluaHookPushArg(lua_State *L, int argt, rpmhookArgv *arg)
+{
+    switch(argt) {
+	case 's':
+	    lua_pushstring(L, arg->s);
+	    break;
+	case 'i':
+	    lua_pushinteger(L, (lua_Integer)arg->i);
+	    break;
+	case 'f':
+	    lua_pushnumber(L, (lua_Number)arg->f);
+	    break;
+	case 'p':
+	    lua_pushlightuserdata(L, arg->p);
+	    break;
+	default:
+	    return -1;
+    }
+    return 0;
+}
+
+static int rpmluaHookGetArg(lua_State *L, int idx, rpmhookArgv *arg)
+{
+    int argt = 0;
+    double f;
+    switch (lua_type(L, idx)) {
+	case LUA_TNIL:
+	    argt = 'p';
+	    arg->p = NULL;
+	    break;
+	case LUA_TNUMBER:
+	    f = (double)lua_tonumber(L, idx);
+	    if (f == (int)f) {
+		argt = 'i';
+		arg->i = (int)f;
+	    } else {
+		argt = 'f';
+		arg->f = f;
+	    }
+	    break;
+	case LUA_TSTRING:
+	    argt = 's';
+	    arg->s = lua_tostring(L, idx);
+	    break;
+	case LUA_TUSERDATA:
+	case LUA_TLIGHTUSERDATA:
+	    argt = 'p';
+	    arg->p = lua_touserdata(L, idx);
+	    break;
+	default:
+	    argt = 0;
+	    arg->p = NULL;
+    }
+    return argt;
+}
+
 static int rpmluaHookWrapper(rpmhookArgs args, void *data)
 {
     rpmluaHookData hookdata = (rpmluaHookData)data;
@@ -560,30 +669,14 @@ static int rpmluaHookWrapper(rpmhookArgs args, void *data)
     lua_rawgeti(L, LUA_REGISTRYINDEX, hookdata->funcRef);
     lua_newtable(L);
     for (i = 0; i != args->argc; i++) {
-	switch (args->argt[i]) {
-	    case 's':
-		lua_pushstring(L, args->argv[i].s);
-		lua_rawseti(L, -2, i+1);
-		break;
-	    case 'i':
-		lua_pushnumber(L, (lua_Number)args->argv[i].i);
-		lua_rawseti(L, -2, i+1);
-		break;
-	    case 'f':
-		lua_pushnumber(L, (lua_Number)args->argv[i].f);
-		lua_rawseti(L, -2, i+1);
-		break;
-	    case 'p':
-		lua_pushlightuserdata(L, args->argv[i].p);
-		lua_rawseti(L, -2, i+1);
-		break;
-	    default:
-                (void) luaL_error(L, "unsupported type '%c' as "
-                              "a hook argument\n", args->argt[i]);
-		break;
+	if (rpmluaHookPushArg(L, args->argt[i], args->argv + i)) {
+	    (void) luaL_error(L, "unsupported type '%c' as "
+			  "a hook argument\n", args->argt[i]);
+	    continue;
 	}
+	lua_rawseti(L, -2, i+1);
     }
-    if (lua_pcall(L, 1, 1, 0) != 0) {
+    if (rpm_pcall(L, 1, 1, 0) != 0) {
 	rpmlog(RPMLOG_ERR, _("lua hook failed: %s\n"),
 		 lua_tostring(L, -1));
 	lua_pop(L, 1);
@@ -640,35 +733,11 @@ static int rpm_call(lua_State *L)
 	char *argt = (char *)xmalloc(args->argc+1);
 	int i;
 	for (i = 0; i != args->argc; i++) {
-	    switch (lua_type(L, i+1)) {
-		case LUA_TNIL:
-		    argt[i] = 'p';
-		    args->argv[i].p = NULL;
-		    break;
-		case LUA_TNUMBER: {
-		    float f = (float)lua_tonumber(L, i+1);
-		    if (f == (int)f) {
-			argt[i] = 'i';
-			args->argv[i].i = (int)f;
-		    } else {
-			argt[i] = 'f';
-			args->argv[i].f = f;
-		    }
-		}   break;
-		case LUA_TSTRING:
-		    argt[i] = 's';
-		    args->argv[i].s = lua_tostring(L, i+1);
-		    break;
-		case LUA_TUSERDATA:
-		case LUA_TLIGHTUSERDATA:
-		    argt[i] = 'p';
-		    args->argv[i].p = lua_touserdata(L, i+1);
-		    break;
-		default:
-		    (void) luaL_error(L, "unsupported Lua type passed to hook");
-		    argt[i] = 'p';
-		    args->argv[i].p = NULL;
-		    break;
+	    argt[i] = rpmluaHookGetArg(L, i + 2, args->argv + i);
+	    if (!argt[i]) {
+		(void) luaL_error(L, "unsupported Lua type passed to hook");
+		argt[i] = 'p';
+		args->argv[i].p = NULL;
 	    }
 	}
 	args->argt = argt;
@@ -682,20 +751,13 @@ static int rpm_call(lua_State *L)
 /* Based on luaB_print. */
 static int rpm_print (lua_State *L)
 {
-    rpmlua lua = (rpmlua)getdata(L, "lua");
+    rpmlua lua = getlua(L);
     int n = lua_gettop(L);  /* number of arguments */
     int i;
     if (!lua) return 0;
-    lua_getglobal(L, "tostring");
     for (i = 1; i <= n; i++) {
-	const char *s;
 	size_t sl;
-	lua_pushvalue(L, -1);  /* function to be called */
-	lua_pushvalue(L, i);   /* value to print */
-	lua_call(L, 1, 1);
-	s = lua_tolstring(L, -1, &sl);  /* get result */
-	if (s == NULL)
-	    return luaL_error(L, "`tostring' must return a string to `print'");
+	const char *s = luaL_tolstring(L, i, &sl);
 	if (lua->printbuf) {
 	    rpmluapb prbuf = lua->printbuf;
 	    if (prbuf->used+sl+1 > prbuf->alloced) {
@@ -742,7 +804,10 @@ static int rpm_redirect2null(lua_State *L)
 	(void) close(fd);
 	errno = e;
     }
-    return pushresult(L, r, NULL);
+    if (r < 0)
+	return pusherror(L, errno, NULL);
+
+    return pushresult(L, r);
 }
 
 static int rpm_exit(lua_State *L)
@@ -771,11 +836,49 @@ static int rpm_execute(lua_State *L)
     status = posix_spawnp(&pid, file, NULL, NULL, argv, environ);
     free(argv);
     if (status != 0)
-	return pusherror(L, status, "posix_spawnp");
+	return pusherror(L, status, NULL);
     if (waitpid(pid, &status, 0) == -1)
-	return pusherror(L, 0, "waitpid");
-    else
-	return pushresult(L, status, NULL);
+	return pusherror(L, errno, NULL);
+    if (status != 0)
+	return pusherror(L, status, "exit code");
+
+    return pushresult(L, status);
+}
+
+static int rpm_splitargs(lua_State *L)
+{
+    const char *str = luaL_checkstring(L, 1);
+    ARGV_t args = NULL;
+    int i;
+
+    splitQuoted(&args, str, " \t");
+    lua_newtable(L);
+    for (i = 0; args && args[i]; i++) {
+	lua_pushstring(L, args[i]);
+	lua_rawseti(L, -2, i + 1);
+    }
+    return 1;
+}
+
+static int rpm_unsplitargs(lua_State *L)
+{
+    char *str;
+    ARGV_t args = NULL;
+    int i;
+
+    luaL_checktype(L, 1, LUA_TTABLE);
+    for (i = 1; i; i++) {
+        lua_rawgeti(L, 1, i);
+        if (lua_type(L, -1) == LUA_TNIL)
+	    i = -1;
+	else
+	    argvAdd(&args, (char *)luaL_checkstring(L, -1));
+	lua_pop(L, 1);
+    }
+    str = args ? unsplitQuoted(args, " ") : NULL;
+    lua_pushstring(L, str ? str : "");
+    free(str);
+    return 1;
 }
 
 static int newinstance(lua_State *L, const char *name, void *p)
@@ -1063,7 +1166,7 @@ static int mc_call(lua_State *L)
     if (lua_isstring(L, 1)) {
 	lua_pushfstring(L, "%%{%s %s}", name, lua_tostring(L, 1));
 	/* throw out previous args and call expand() with our result string */
-	lua_rotate(L, 1, 1);
+	lua_replace(L, 1);
 	lua_settop(L, 1);
 	rc = rpm_expand(L);
     } else if (lua_istable(L, 1)) {
@@ -1073,14 +1176,21 @@ static int mc_call(lua_State *L)
 
 	for (int i = 1; i <= nitem; i++) {
 	    lua_rawgeti(L, 1, i);
-	    argvAdd(&argv, lua_tostring(L, -1));
-	    lua_pop(L, 1);
+	    const char *s= lua_tostring(L, -1);
+	    if (s) {
+		argvAdd(&argv, s);
+		lua_pop(L, 1);
+	    } else {
+		luaL_argerror(L, i, "cannot convert to string");
+	    }
 	}
 
 	if (rpmExpandThisMacro(*mc, name, argv, &buf, 0) >= 0) {
 	    rc = 1;
 	    lua_pushstring(L, buf);
 	    free(buf);
+	} else {
+	    luaL_error(L, "error expanding macro");
 	}
 	argvFree(argv);
     } else {
@@ -1103,7 +1213,7 @@ static int mc_index(lua_State *L)
 	    rc = 1;
 	} else {
 	    lua_pushfstring(L, "%%{%s}", a);
-	    lua_rotate(L, 1, 1);
+	    lua_replace(L, 1);
 	    lua_settop(L, 1);
 	    rc = rpm_expand(L);
 	}
@@ -1154,12 +1264,13 @@ static const luaL_Reg rpmlib[] = {
     {"unregister", rpm_unregister},
     {"call", rpm_call},
     {"interactive", rpm_interactive},
-    {"next_file", rpm_next_file},
     {"execute", rpm_execute},
     {"redirect2null", rpm_redirect2null},
     {"vercmp", rpm_vercmp},
     {"ver", rpm_ver_new},
     {"open", rpm_open},
+    {"splitargs", rpm_splitargs},
+    {"unsplitargs", rpm_unsplitargs},
     {NULL, NULL}
 };
 
