@@ -576,32 +576,38 @@ static int pgpCurveByOid(const uint8_t *p, int l)
     return 0;
 }
 
+static int isKey(pgpDigParams keyp)
+{
+    return keyp->tag == PGPTAG_PUBLIC_KEY || keyp->tag == PGPTAG_PUBLIC_SUBKEY;
+}
+
 static int pgpPrtPubkeyParams(uint8_t pubkey_algo,
 		const uint8_t *p, const uint8_t *h, size_t hlen,
 		pgpDigParams keyp)
 {
-    int rc = 1;
+    int rc = 1; /* assume failure */
     const uint8_t *pend = h + hlen;
     int curve = 0;
+    if (!isKey(keyp))
+	return rc;
+    /* We can't handle more than one key at a time */
+    if (keyp->alg)
+	return rc;
     if (pubkey_algo == PGPPUBKEYALGO_EDDSA) {
 	int len = (hlen > 1) ? p[0] : 0;
 	if (len == 0 || len == 0xff || len >= hlen)
-	    goto exit;
+	    return rc;
 	curve = pgpCurveByOid(p + 1, len);
 	p += len + 1;
     }
     pgpDigAlg keyalg = pgpPubkeyNew(pubkey_algo, curve);
     rc = processMpis(keyalg->mpis, keyalg, p, pend);
-
-    /* We can't handle more than one key at a time */
-    if (rc == 0 && keyp->alg == NULL && (keyp->tag == PGPTAG_PUBLIC_KEY ||
-	keyp->tag == PGPTAG_PUBLIC_SUBKEY))
-
+    if (rc == 0) {
+	keyp->pubkey_algo = pubkey_algo;
 	keyp->alg = keyalg;
-    else
+    } else {
 	pgpDigAlgFree(keyalg);
-
-exit:
+    }
     return rc;
 }
 
@@ -631,7 +637,6 @@ static int pgpPrtKey(pgpTag tag, const uint8_t *h, size_t hlen,
 		_digp->version = v->version;
 		if (!(_digp->saved & PGPDIG_SAVED_TIME))
 		    _digp->time = pgpGrab(v->time, sizeof(v->time));
-		_digp->pubkey_algo = v->pubkey_algo;
 		_digp->saved |= PGPDIG_SAVED_TIME | PGPDIG_SIG_HAS_CREATION_TIME;
 	    }
 
@@ -1037,7 +1042,8 @@ int pgpPrtParams(const uint8_t * pkts, size_t pktlen, unsigned int pkttype,
 	    } else {
 		digp = pgpDigParamsNew(pkt->tag);
 	    }
-	}
+	} else if (pkt->tag == PGPTAG_PUBLIC_KEY)
+	    break;
 
 	if (expect) {
 	    if (pkt->tag != expect)
@@ -1200,6 +1206,9 @@ rpmRC pgpVerifySignature(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx)
     if (sig == NULL || ctx == NULL)
 	goto exit;
 
+    if (sig->tag != PGPTAG_SIGNATURE)
+	goto exit;
+
     if (sig->hash != NULL)
 	rpmDigestUpdate(ctx, sig->hash, sig->hashlen);
 
@@ -1226,9 +1235,11 @@ rpmRC pgpVerifySignature(pgpDigParams key, pgpDigParams sig, DIGEST_CTX hashctx)
      * done all we can, return NOKEY to indicate "looks okay but dunno."
      */
     if (key && key->alg) {
+	if (!isKey(key))
+	    goto exit;
 	pgpDigAlg sa = sig->alg;
 	pgpDigAlg ka = key->alg;
-	if (sa && sa->verify) {
+	if (sa && sa->verify && sig->pubkey_algo == key->pubkey_algo) {
 	    if (sa->verify(ka, sa, hash, hashlen, sig->hash_algo) == 0) {
 		res = RPMRC_OK;
 	    }
