@@ -140,6 +140,7 @@ static rpmtd makeSigTag(Header sigh, int ishdr, uint8_t *pkt, size_t pktlen)
     rpmtd sigtd = NULL;
     unsigned int hash_algo;
     unsigned int pubkey_algo;
+    int ver;
 
     if (pgpPrtParams(pkt, pktlen, PGPTAG_SIGNATURE, &sigp)) {
 	rpmlog(RPMLOG_ERR, _("Unsupported PGP signature\n"));
@@ -168,6 +169,12 @@ static rpmtd makeSigTag(Header sigh, int ishdr, uint8_t *pkt, size_t pktlen)
 	break;
     }
 
+    ver = pgpDigParamsVersion(sigp);
+    if (ver < 4) {
+	rpmlog(RPMLOG_WARNING, _("Deprecated OpenPGP signature version %d\n"),
+		ver);
+    }
+
     /* Looks sane, create the tag data */
     sigtd = rpmtdNew();
     sigtd->count = pktlen;
@@ -632,14 +639,20 @@ static int rpmSign(const char *rpm, int deleting, int flags)
 	res = -1;
     }
 
-    /* Try to make new signature smaller to have size of original signature */
+    /* Adjust reserved size for added/removed signatures */
     if (headerGet(sigh, RPMSIGTAG_RESERVEDSPACE, &utd, HEADERGET_MINMEM)) {
 	int diff = headerSizeof(sigh, HEADER_MAGIC_YES) - origSigSize;
 
-	if (diff > 0 && diff < utd.count) {
+	/* diff can be zero if nothing was added or removed */
+	if (diff) {
 	    utd.count -= diff;
-	    headerMod(sigh, &utd);
-	    insSig = 1;
+	    if (utd.count > 0 && utd.count < origSigSize) {
+		char *zeros = xcalloc(utd.count, sizeof(*zeros));
+		utd.data = zeros;
+		headerMod(sigh, &utd);
+		insSig = 1;
+		free(zeros);
+	    }
 	}
     }
 
@@ -648,14 +661,14 @@ static int rpmSign(const char *rpm, int deleting, int flags)
     if (sigh == NULL)	/* XXX can't happen */
 	goto exit;
 
+    if (Fseek(fd, insSig ? sigStart : 0, SEEK_SET) < 0) {
+	rpmlog(RPMLOG_ERR, _("Could not seek in file %s: %s\n"),
+		rpm, Fstrerror(fd));
+	goto exit;
+    }
+
     if (insSig) {
 	/* Insert new signature into original rpm */
-	if (Fseek(fd, sigStart, SEEK_SET) < 0) {
-	    rpmlog(RPMLOG_ERR, _("Could not seek in file %s: %s\n"),
-		    rpm, Fstrerror(fd));
-	    goto exit;
-	}
-
 	if (rpmWriteSignature(fd, sigh)) {
 	    rpmlog(RPMLOG_ERR, _("%s: rpmWriteSignature failed: %s\n"), rpm,
 		Fstrerror(fd));
@@ -663,6 +676,7 @@ static int rpmSign(const char *rpm, int deleting, int flags)
 	}
 	res = 0;
     } else {
+	char lbuf[RPMLEAD_SIZE];
 	/* Replace orignal rpm with new rpm containing new signature */
 	rasprintf(&trpm, "%s.XXXXXX", rpm);
 	ofd = rpmMkTemp(trpm);
@@ -671,11 +685,12 @@ static int rpmSign(const char *rpm, int deleting, int flags)
 	    goto exit;
 	}
 
-	/* Write the lead/signature of the output rpm */
-	rc = rpmLeadWrite(ofd, h);
-	if (rc != RPMRC_OK) {
-	    rpmlog(RPMLOG_ERR, _("%s: writeLead failed: %s\n"), trpm,
-		Fstrerror(ofd));
+	/* Copy lead from original */
+	if (!(Fread(lbuf, 1, RPMLEAD_SIZE, fd) == RPMLEAD_SIZE &&
+	      Fwrite(lbuf, 1, RPMLEAD_SIZE, ofd) == RPMLEAD_SIZE))
+	{
+	    rpmlog(RPMLOG_ERR, _("%s: failed to copy rpm lead: %s\n"),
+				trpm, Fstrerror(ofd));
 	    goto exit;
 	}
 
